/*
 * Copyright (C) 1999 Dirk-Willem van Gulik, dirkx@webweaving.org
 *      WebWeaving Consultancy, All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 *
 * THIS SOFTWARE IS PROVIDED BY WEBWEAVING AND CONTRIBUTORS ``AS IS'' AND
 * ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED.  IN NO EVENT SHALL WEBWEAVING OR CONTRIBUTORS BE LIABLE
 * FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
 * DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS
 * OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
 * LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY
 * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 *
 * $FreeBSD$
 *
 * Just a simple example to test a really cool USB (and not to mention low
 * cost) i/o board having 16 i/o's and a i2c port.
 *
 * ActiveWire Inc. USB board. See
 *      http://www.activewireinc.com
 */

#include <stdlib.h>
#include <stdio.h>
#include <sys/errno.h>
#include <string.h>
#include <sys/fcntl.h>
#include <sys/ioctl.h>
#include <unistd.h>

extern char * optarg;
extern int opterr;

#include <dev/usb/usb.h>

#define DEVICENAME	"/dev/ugen0"
int
main(int argc, char **argv)
{
	int fw,fd,i,endpoint,debug=0,len=2;
	char ch=0,fn[128],*device = DEVICENAME;
	unsigned char x[ len ];
	struct usb_device_info udi;

	if (argc == 2)
		device = argv[1];
	else 
	if (argc != 1) {
		fprintf(stderr,"Duh: %s [device]\n",argv[0]);
		exit(1);
	};

	/* open the master device to check that it
	 * actually is an active wire board
	 */
	snprintf(fn,127,"%s%s", 
		(device[0] == '/') ? "" : "/dev/", device);

        fd = open(fn, O_RDONLY, 0);
        if ( fd == -1) {
                fprintf(stderr, "Open usb endpoint '%s' failed: %s\n",
                        fn, strerror(errno));
                return(1);
        };

        if (ioctl(fd, USB_GET_DEVICEINFO, &udi)) {
                perror("Could not get information on this device");
                return (1);
        };
#define SURE(x) \
	(((x!=NULL) && (*x !='\0')) ? x : "<not defined>")

        printf("Device information: vendor %04x (%s) product %04x (%s) rev %s addr %x\n",
                        udi.vendorNo, SURE(udi.vendor), 
                        udi.productNo, SURE(udi.product),
                        SURE(udi.release), udi.addr);
	close(fd);

	/* vendor 0854 (ActiveWire, Inc.) product 0101 (ActiveWire USB) rev 0.00 
	 */
	if ((udi.vendorNo != 0x854) || (udi.productNo != 0x101)) {
		fprintf(stderr,"Sorry, does not seem to be an ActiveWire board. Aborted\n");
		exit(1);
	}
#if 0
	if (!strcmp(udi.revision,"0.00")) {
		fprintf(stderr,"Looks like an ActiveWire board, but without the firmware. Did\n"
			"you download that using ezdownload(8) ?\n"
			);
		exit(1);
	}
#endif	
	/* Construct a device string to an arbitrary endpoint
	 * of the board. In this case endpoint 2, see firmware
	 * documentation on www.activewireinc.com. This is
	 * the endpoint which determines dataflow direction. 
	 * a '1' at bit location 3 means that bit one will
	 * be an output rep. input.
	 */
	endpoint = 2;
	snprintf(fn,127,"%s%s%s%d",
		(device[0] == '/') ? "" : "/dev/",
		device,
		(endpoint >0) ? "." : "\0",
		endpoint
		);

	fd = open(fn, O_WRONLY, 0);
	if ( fd == -1) {
		fprintf(stderr, "Open(%s) returned: %s\n",
			fn, strerror(errno));
		return(1);
	} else
	if (debug)
		printf("Endpoint %s opened\n",fn);

	/* all 16 bits to '1', i.e. all
	 * are outputs now.
	 */
	memset(x,255,len);

	/* and set the outputs */
	write(fd,&x,len);
	close(fd);

	/* Construct the device name of the .1
	 * end point; where you can read/write
	 * data to.
	 */
	endpoint = 1;
	snprintf(fn,127,"%s%s%s%d",
		(device[0] == '/') ? "" : "/dev/",
		device,
		(endpoint >0) ? "." : "\0",
		endpoint
		);
	
	/* open for write */
	fd = open(fn, O_WRONLY, 0);
	if ( fd == -1) {
		fprintf(stderr, "Open(%s) returned: %s\n",
			fn, strerror(errno));
		return(1);
	} else
	if (debug)
		printf("Endpoint %s opened for write\n",fn);

	/* and for read.. */
	fw = open(fn, O_RDONLY, 0);
	if ( fw == -1) {
		fprintf(stderr, "Open(%s) returned: %s\n",
			fn, strerror(errno));
		return(1);
	} else
	if (debug)
		printf("Endpoint %s opened for read\n",fn);

	/* and just have some fun by reading/writing to it
	 */
	while(1) {
		unsigned char x[ len ];
		memset(x,ch++,len);

		printf(" tick "); fflush(stdout);
		write(fd,&x,len);

#define	DD	350000
		usleep(DD);

		/* check it.. */
		i=read(fw,&x,len);
		if (i==len) 
			for(i=0;i<sizeof(x);i++)
				printf("%s%02x",i ? "." : "",x[i]);
		else if (i==0)
			printf("<no read>");
		else
			printf("<fail:%s>",strerror(errno));

		memset(x,ch++,len);
		printf(" tok "); fflush(stdout);
		write(fd,&x,len);
		usleep(DD);

		/* check it.. */
		i=read(fw,&x,len); 
		if (i==len) 
			for(i=0;i<sizeof(x);i++)
				printf("%s%02x",i ? "." : "",x[i]);
		else if (i==0)
			printf("<no read>");
		else
			printf("<fail:%s>",strerror(errno));

	};
	close(fd);
	/* NOT REACHED */
	return(0);
};
